#!/usr/bin/env python
# encoding: utf-8

import sys
import hashlib
from Naked.toolshed.file import FileReader
from Naked.toolshed.system import file_exists, stdout, stderr

#------------------------------------------------------------------------------
# Hasher class - generate MD5 and SHA hash digests from files
#------------------------------------------------------------------------------
class Hasher(object):
    def __init__(self):
        pass

    #------------------------------------------------------------------------------
    # PUBLIC METHODS
    #------------------------------------------------------------------------------
    def md5(self, filepath):
        data = self._read_file(filepath)
        return hashlib.md5(data).hexdigest()

    def sha1(self, filepath):
        data = self._read_file(filepath)
        return hashlib.sha1(data).hexdigest()

    def sha224(self, filepath):
        data = self._read_file(filepath)
        return hashlib.sha224(data).hexdigest()

    def sha256(self, filepath):
        data = self._read_file(filepath)
        return hashlib.sha256(data).hexdigest()

    def sha384(self, filepath):
        data = self._read_file(filepath)
        return hashlib.sha384(data).hexdigest()

    def sha512(self, filepath):
        data = self._read_file(filepath)
        return hashlib.sha512(data).hexdigest()

    #------------------------------------------------------------------------------
    # PRIVATE METHODS
    #------------------------------------------------------------------------------
    def _read_file(self, filepath):
        fr = FileReader(filepath)
        return fr.read_bin()


#------------------------------------------------------------------------------
# HashChecker class - compares hash digests by
#   - digest vs. digest
#   - digest vs. file
#   - file vs. file
#------------------------------------------------------------------------------
class HashChecker(object):
    def __init__(self):
        self.files = 0
        self.hashes = 0
        self.error = False
        self.error_number = 0
        self.error_one = ""
        self.error_two = ""
        self.filepath_one = ""
        self.filepath_two = ""
        self.hash_one = ""
        self.hash_two = ""
        self.hash_type = ""

    #------------------------------------------------------------------------------
    # PUBLIC METHOD
    #------------------------------------------------------------------------------
    def compare(self, arg_list):
        for argument in arg_list:
            if file_exists(argument):
                if self.files == 0:
                    self.filepath_one = argument
                else:
                    self.filepath_two = argument
                self.files += 1
            elif self.is_hash(argument):
                if self.hashes == 0:
                    self.hash_one = argument
                else:
                    self.hash_two = argument
                self.hashes += 1
            else:
                self.error = True
                if self.error_number == 0:
                    self.error_one = argument
                else:
                    self.error_two = argument
                self.error_number += 1

        if self.error == True:
            if self.error_number == 1:
                stderr(" ")
                stderr(self.error_one + " does not appear to be a file path or supported hash digest.  Please try again.")
                sys.exit(1)
            else:
                stderr(" ")
                stderr(self.error_one + " does not appear to be a file path or supported hash digest.")
                stderr(self.error_two + " does not appear to be a file path or supported hash digest.")
                stderr("Please try again.")
                sys.exit(1)
        else:
            if self.hashes == 2 and self.files == 0:
                if self.hash_to_hash() == True:
                    stdout(" ")
                    stdout("The hash digests are identical.")
                else:
                    stdout(" ")
                    stdout(self.hash_one)
                    stdout(self.hash_two)
                    stdout(self.differ(self.hash_one, self.hash_two))
                    stdout(" ")
                    stdout("The hash digests are NOT identical.")
            elif self.hashes == 1 and self.files == 1:
                result_dict = self.file_to_hash()
                stdout(result_dict['type'] + " (" + self.filepath_one + ") :")
                stdout(result_dict['filehash'])
                stdout(result_dict['type'] + " (test) :")
                stdout(self.hash_one)
                if result_dict['is_equal'] == True:
                    stdout(" ")
                    stdout("The hash digests are identical.")
                else:
                    stdout(self.differ(result_dict['filehash'], self.hash_one))
                    stdout(" ")
                    stdout("The hash digests are NOT identical.")
            elif self.hashes == 0 and self.files == 2:
                result_dict = self.file_to_file()
                stdout(result_dict['type'] + " (" + self.filepath_one + ") :")
                stdout(result_dict['filehash1'])
                stdout(result_dict['type'] + " (" + self.filepath_two + ") :")
                stdout(result_dict['filehash2'])
                if result_dict['is_equal'] == True:
                    stdout(" ")
                    stdout("The hash digests are identical.")
                else:
                    stdout(self.differ(result_dict['filehash1'], result_dict['filehash2']))
                    stdout(" ")
                    stdout("The hash digests are NOT identical.")

    #------------------------------------------------------------------------------
    # PRIVATE METHODS
    #------------------------------------------------------------------------------

    # determine whether the string is an expected hash digest length, assume the algorithm type based upon length
    def is_hash(self, test_hash):
        hash_length = len(test_hash)

        if hash_length == 32:
            self.hash_type = "md5"
            return True
        elif hash_length == 40:
            self.hash_type = "sha1"
            return True
        elif hash_length == 56:
            self.hash_type = "sha224"
            return True
        elif hash_length == 64:
            self.hash_type = "sha256"
            return True
        elif hash_length == 96:
            self.hash_type = "sha384"
            return True
        elif hash_length == 128:
            self.hash_type = "sha512"
            return True
        else:
            return False

    # compare hash digest CL argument to hash digest CL argument (direct string comparison)
    def hash_to_hash(self):
        if self.hash_one == self.hash_two:
            return True
        else:
            return False

    # compare hash digest generated from file CL argument to a test hash digest string CL argument
    def file_to_hash(self):
        # create Hasher object to read file and generate appropriate hash digest
        hasher = Hasher()

        if self.hash_type == "md5":
            hash_digest = hasher.md5(self.filepath_one)
            return {'type': 'MD5', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}
        elif self.hash_type == "sha1":
            hash_digest = hasher.sha1(self.filepath_one)
            return {'type': 'SHA1', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}
        elif self.hash_type == "sha224":
            hash_digest = hasher.sha224(self.filepath_one)
            return {'type': 'SHA224', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}
        elif self.hash_type == "sha256":
            hash_digest = hasher.sha256(self.filepath_one)
            return {'type': 'SHA256', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}
        elif self.hash_type == "sha384":
            hash_digest = hasher.sha384(self.filepath_one)
            return {'type': 'SHA384', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}
        elif self.hash_type == "sha512":
            hash_digest = hasher.sha512(self.filepath_one)
            return {'type': 'SHA512', 'filehash': hash_digest, 'is_equal': hash_digest == self.hash_one}

    # compare file CL argument to file CL argument by generating SHA256 hash digest for each file and comparing
    def file_to_file(self):
        hasher = Hasher()
        hash_digest_one = hasher.sha256(self.filepath_one)
        hash_digest_two = hasher.sha256(self.filepath_two)

        return {'type': 'SHA256', 'filehash1': hash_digest_one, 'filehash2': hash_digest_two, 'is_equal': hash_digest_one == hash_digest_two}

    # create a diff indicator for the non-matched positions on the hash strings (called from compare() function above)
    def differ(self, file_hash, test_hash):
        diff_list = []
        i = 0
        if len(file_hash) == len(test_hash):
            for x in file_hash:
                if test_hash[i] == x:
                    diff_list.append('=')
                else:
                    diff_list.append('^')
                i += 1
            return ''.join(diff_list)
        else:
            return " "







#!/usr/bin/env python
# encoding: utf-8

#------------------------------------------------------------------------------
# Application Name
#------------------------------------------------------------------------------
app_name = 'hsh'

#------------------------------------------------------------------------------
# Version Number
#------------------------------------------------------------------------------
major_version = "1"
minor_version = "0"
patch_version = "1"

#------------------------------------------------------------------------------
# Debug Flag (switch to False for production release code)
#------------------------------------------------------------------------------
debug = False

#------------------------------------------------------------------------------
# Usage String
#------------------------------------------------------------------------------
usage = """
REPORT HASH DIGEST
  hsh [filepath]
  hsh <command> [filepath]

FILE INTEGRITY CHECKS
  hsh [hash digest string 1] [hash digest string 2]
  hsh [filepath] [hash digest string]
  hsh [filepath 1] [filepath 2]
"""

#------------------------------------------------------------------------------
# Help String
#------------------------------------------------------------------------------
help = """
-------------------------------------------------
hsh
Simple file hash digests & file integrity checks
Copyright 2015 Christopher Simpkins
MIT license
Source: https://github.com/chrissimpkins/hsh
Docs: http://chrissimpkins.github.io/hsh/
-------------------------------------------------

ABOUT
  hsh generates MD5, SHA1, SHA224, SHA256, SHA384, and SHA512 hash digests for files and performs file integrity checks based upon hash digest string to hash digest string, file path to hash digest string, and file path to file path comparisons.

USAGE
  REPORT HASH DIGEST
    hsh [filepath]
    hsh <command> [filepath]

  FILE INTEGRITY CHECKS
    hsh [hash digest string 1] [hash digest string 2]
    hsh [filepath] [hash digest string]
    hsh [filepath 1] [filepath 2]

COMMANDS
    md5            report MD5 hash digest for file
    sha1           report SHA1 hash digest for file
    sha224         report SHA224 hash digest for file
    sha256         report SHA256 hash digest for file
    sha384         report SHA384 hash digest for file
    sha512         report SHA512 hash digest for file

OPTIONS
    -h --help      display help documentation
       --usage     display usage documentation
    -v --version   display application version

DESCRIPTION
  Hash digests are generated with the SHA256 algorithm by default. To modify this, add an explicit primary command argument (see COMMANDS above).

  Hash digest comparisons are performed with any file path and/or file hash digest string argument combination.  hsh supports comparisons of MD5, SHA1, SHA224, SHA256, SHA384, and SHA512 hash digests.  When you include a file hash digest string to compare with a file on a local file path, hsh generates the appropriate hash digest for the file path argument based upon the length of your file hash digest string argument.  When the hash digests differ, a diff indicator is displayed below the hash digest strings.  The = character indicates matching positions and the ^ character indicates positions that do not match.
"""



#!/usr/bin/env python
# encoding: utf-8

#------------------------------------------------------------------------------
# hsh
# Copyright 2015 Christopher Simpkins
# MIT license
#------------------------------------------------------------------------------

# Application start
def main():
    import sys
    from hsh.library.hash import Hasher, HashChecker
    from Naked.commandline import Command
    from Naked.toolshed.file import FileReader
    from Naked.toolshed.system import file_exists, dir_exists, stdout, stderr

    #------------------------------------------------------------------------------------------
    # [ Instantiate command line object ]
    #   used for all subsequent conditional logic in the CLI application
    #------------------------------------------------------------------------------------------
    c = Command(sys.argv[0], sys.argv[1:])
    #------------------------------------------------------------------------------------------
    # [ Command Suite Validation ] - early validation of appropriate command syntax
    # Test that user entered at least one argument to the executable, print usage if not
    #------------------------------------------------------------------------------------------
    if not c.command_suite_validates():
        from hsh.settings import usage as hsh_usage
        print(hsh_usage)
        sys.exit(1)
    #------------------------------------------------------------------------------------------
    # [ NAKED FRAMEWORK COMMANDS ]
    # Naked framework provides default help, usage, and version commands for all applications
    #   --> settings for user messages are assigned in the lib/hsh/settings.py file
    #------------------------------------------------------------------------------------------
    if c.help():      # User requested hsh help information
        from hsh.settings import help as hsh_help
        print(hsh_help)
        sys.exit(0)
    elif c.usage():   # User requested hsh usage information
        from hsh.settings import usage as hsh_usage
        print(hsh_usage)
        sys.exit(0)
    elif c.version(): # User requested hsh version information
        from hsh.settings import app_name, major_version, minor_version, patch_version
        version_display_string = app_name + ' ' + major_version + '.' + minor_version + '.' + patch_version
        print(version_display_string)
        sys.exit(0)
    #------------------------------------------------------------------------------------------
    # [ PRIMARY COMMAND LOGIC ]
    #   Enter your command line parsing logic below
    #------------------------------------------------------------------------------------------

    primary_command = c.cmd.lower() #convert to lowercase to support user entry of upper case (e.g. SHA256) as primary command

    if primary_command == "sha1":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.sha1(file)
                    stdout("SHA1 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "sha224":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.sha224(file)
                    stdout("SHA224 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "sha256":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.sha256(file)
                    stdout("SHA256 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "sha384":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.sha384(file)
                    stdout("SHA384 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "sha512":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.sha512(file)
                    stdout("SHA512 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "md5":
        if c.argc > 1:
            file_list = c.argv[1:]
            for file in file_list:
                if file_exists(file):
                    hasher = Hasher()
                    sha_hash = hasher.md5(file)
                    stdout("MD5 (" + file + ") :")
                    stdout(sha_hash)
                else:
                    stderr(file + " does not appear to be an existing file path.")
        else:
            stderr("You did not include a file in your command.  Please try again.")
    elif primary_command == "check":
        if c.argc == 3: # primary command + 2 arguments
            hc = HashChecker()
            hc.compare(c.argv[1:]) # pass the argument list excluding the primary command
        elif c.argc < 3:
            stderr("You did not include a file or hash digest for comparison.  Please try again.")
        elif c.argc > 3:
            stderr("Too many arguments.  Please include two arguments for comparison.")
    elif c.argc == 1: # single file hash digest request with default SHA256 settings
        file = c.arg0
        if file_exists(file):
            hasher = Hasher()
            sha_hash = hasher.sha256(file)
            stdout("SHA256 (" + file + ") :")
            stdout(sha_hash)
        else:
            stderr(c.arg0 + " does not appear to be an existing file path. Please try again.")
    elif c.argc == 2: # exactly two arguments, perform comparison between them by default
        hc = HashChecker()
        hc.compare(c.argv) # pass the entire argument list because there is no primary command

    #------------------------------------------------------------------------------------------
    # [ DEFAULT MESSAGE FOR MATCH FAILURE ]
    #  Message to provide to the user when all above conditional logic fails to meet a true condition
    #------------------------------------------------------------------------------------------
    else:
        print("Could not complete the command that you entered.  Please try again.")
        sys.exit(1) #exit


if __name__ == '__main__':
    main()
